@@ -5,9 +5,15 @@
 #' from the the epsilon-greedy policy and then updated using observations.
 #'
 #' Simulates `n` trajectories.
-#' If no simulation horizon is specified the horizon of finite-horizon problems 
+#' If no simulation horizon is specified, the horizon of finite-horizon problems 
 #' is used. For infinite-horizon problems with \eqn{\gamma < 1}, the simulation
-#' horizon \eqn{T} is chosen such that \deqn{abs(\gamma^T R_\text{max}) \le \delta_\text{horizon}.}
+#' horizon \eqn{T} is chosen such that 
+#' the worst-case error is no more than \eqn{\delta_\text{horizon}}. That is
+#' 
+#' \deqn{\gamma^T \frac{R_\text{max}}{\gamma} \le \delta_\text{horizon},}
+#' 
+#' where \eqn{R_\text{max}} is the largest possible absolute reward value used as a
+#' perpetuity starting after \eqn{T}.
 #' 
 #' A native R implementation (`engine = 'r'`) and a faster C++ implementation
 #' (`engine = 'cpp'`) are available. Currently, only the R implementation supports 
@@ -31,21 +37,25 @@
 #' @param horizon number of epochs for the simulation. If `NULL` then the
 #'  horizon for finite-horizon model is used. For infinite-horizon problems, a horizon is 
 #'  calculated using the discount factor.
-#' @param return_beliefs logical; Return all visited belief states? This requires n x horizon memory.
 #' @param epsilon the probability of random actions for using an epsilon-greedy policy.
 #'  Default for solved models is 0 and for unsolved model 1.
 #' @param delta_horizon precision used to determine the horizon for infinite-horizon problems.
 #' @param digits round probabilities for belief points.
+#' @param return_beliefs logical; Return all visited belief states? This requires n x horizon memory.
+#' @param return_trajectories logical; Return the simulated trajectories as a data.frame?
 #' @param engine `'cpp'`, `'r'` to perform simulation using a faster C++ or a 
 #'  native R implementation.
 #' @param verbose report used parameters.
 #' @param ... further arguments are ignored.
 #' @return A list with elements:
 #'  * `avg_reward`: The average discounted reward.
-#'  * `belief_states`: A matrix with belief states as rows.
 #'  * `action_cnt`: Action counts.
 #'  * `state_cnt`: State counts.
 #'  * `reward`: Reward for each trajectory.
+#'  * `belief_states`: A matrix with belief states as rows.
+#'  * `trajectories`: A data.frame with the `episode` id, `time`, the state of the 
+#'    simulation (`simulation_state`), the id of the used alpha vector given the current belief 
+#'    (see `belief_states` above), the action `a` and the reward `r`.
 #' @author Michael Hahsler
 #' @md
 #' @examples
@@ -62,7 +72,7 @@
 #' # doParallel::registerDoParallel()
 #' # foreach::getDoParWorkers()
 #'
-#' ## Example 1: simulate 10 trajectories
+#' ## Example 1: simulate 100 trajectories
 #' sim <- simulate_POMDP(sol, n = 100, verbose = TRUE)
 #' sim
 #'
@@ -72,9 +82,12 @@
 #' # reward distribution
 #' hist(sim$reward)
 #'
-#' ## Example 2: look at all belief states in the trajectory starting with an initial start belief.
-#' sim <- simulate_POMDP(sol, n = 100, belief = c(.5, .5), return_beliefs = TRUE)
+#' ## Example 2: look at the belief states and the trajectories starting with 
+#' #             an initial start belief.
+#' sim <- simulate_POMDP(sol, n = 100, belief = c(.5, .5), 
+#'   return_beliefs = TRUE, return_trajectories = TRUE)
 #' head(sim$belief_states)
+#' head(sim$trajectories)
 #'
 #' # plot with added density (the x-axis is the probability of the second belief state)
 #' plot_belief_space(sol, sample = sim$belief_states, jitter = 2, ylim = c(0, 6))
@@ -83,10 +96,12 @@
 #'
 #' ## Example 3: simulate trajectories for an unsolved POMDP which uses an epsilon of 1
 #' #             (i.e., all actions are randomized). The simulation horizon for the 
-#' #             infinite-horizon Tiger problem is calculated. 
-#' sim <- simulate_POMDP(Tiger, n = 100, return_beliefs = TRUE, verbose = TRUE)
+#' #             infinite-horizon Tiger problem is calculated using delta_horizon. 
+#' sim <- simulate_POMDP(Tiger, return_beliefs = TRUE, verbose = TRUE)
 #' sim$avg_reward
-#'
+#' 
+#' hist(sim$reward, breaks = 20)
+#' 
 #' plot_belief_space(sol, sample = sim$belief_states, jitter = 2, ylim = c(0, 6))
 #' lines(density(sim$belief_states[, 1], bw = .05)); axis(2); title(ylab = "Density")
 #' @export
@@ -95,16 +110,29 @@ simulate_POMDP <-
     n = 1000,
     belief = NULL,
     horizon = NULL,
-    return_beliefs = FALSE,
     epsilon = NULL,
     delta_horizon = 1e-3,
     digits = 7L,
+    return_beliefs = FALSE,
+    return_trajectories = FALSE,
     engine = "cpp",
     verbose = FALSE,
     ...) {
     time_start <- proc.time()  
-    
+     
     engine <- match.arg(tolower(engine), c("cpp", "r"))
+    if(engine == "cpp" && 
+       (is.function(model$transition_prob) ||
+       is.function(model$observation_prob) ||
+       is.function(model$reward))) {
+      
+      warning("Some elements of the POMDP are defined as R funciton. The CPP engine is very slow with R function calls.\n",
+              "Falling back to R. Normalize the model first to use CPP.", immediate. = TRUE
+              )
+      engine <- 'r'
+    }
+    
+    
     
     solved <- is_solved_POMDP(model)
     dt <- is_timedependent_POMDP(model)
@@ -121,10 +149,13 @@ simulate_POMDP <-
       if (is.null(model$discount) || !(model$discount < 1)) 
         stop("Simulation needs a finite simulation horizon.")
       
-      # find a horizon that approximates the reward using 
+      # find a horizon that approximates the reward with a maximal error
+      # of delta. The PV of the perpetuity of max_abs_R starting after t at gamma is 
+      # PV = max_abs_R / gamma * gamma^t. For the PV <= delta, we need to solve
+      # t >= log(delta/max_abs_R) / log(gamma) + 1
       # discount^horizon * max_abs_R <= 0.001
-      max_abs_R <-  max(abs(reward_matrix(model, sparse = TRUE)$value))
-      horizon <- ceiling(log(delta_horizon/max_abs_R)/log(model$discount))
+      max_abs_R <- .max_abs_reward(model)
+      horizon <- ceiling(log(delta_horizon/max_abs_R)/log(model$discount)) + 1
     }
     horizon <- as.integer(horizon)
     
@@ -143,9 +174,12 @@ simulate_POMDP <-
       disc <- 1
     
     if (engine == "cpp") {
+      
       if (!dt) {
         ### TODO: Add support for time dependence
-        model <- normalize_POMDP(model, sparse = TRUE)
+        
+        # Cpp can now deal with unnormalized POMDPs
+        #model <- normalize_POMDP(model, sparse = TRUE)
         
         if (foreach::getDoParWorkers() == 1 || n * horizon < 100000L) {
           res <- simulate_POMDP_cpp(
@@ -155,6 +189,7 @@ simulate_POMDP <-
             horizon,
             disc,
             return_beliefs,
+            return_trajectories,
             epsilon,
             digits,
             verbose
@@ -197,10 +232,19 @@ simulate_POMDP <-
             horizon,
             disc,
             return_beliefs,
+            return_trajectories,
             epsilon,
             digits,
             verbose = FALSE)
         
+        
+        # adjust the episode number for parallel processing
+        episode_add <- cumsum(c(0L, ns))
+        for (i in seq_along(sim)) {
+          sim[[i]]$trajectories$episode <-
+            sim[[i]]$trajectories$episode + episode_add[i]
+        }
+        
         rew <- Reduce(c,  lapply(sim, "[[", "reward"))
     
         time_end <- proc.time()
@@ -214,7 +258,8 @@ simulate_POMDP <-
             action_cnt = Reduce('+', lapply(sim, "[[" , "action_cnt")),
             state_cnt =  Reduce('+', lapply(sim, "[[", "state_cnt")),
             obs_cnt =    Reduce('+', lapply(sim, "[[", "obs_cnt")),
-            belief_states = Reduce(rbind, lapply(sim, "[[", "belief_states"))
+            belief_states = Reduce(rbind, lapply(sim, "[[", "belief_states")),
+            trajectories = Reduce(rbind, lapply(sim, "[[", "trajectories"))
           )
         )
         
@@ -224,8 +269,11 @@ simulate_POMDP <-
       }
     }
     
+    ##### use R ############
+    
     states <- as.character(model$states)
     n_states <- length(states)
+    states_absorbing <- which(absorbing_states(model))
     obs <- as.character(model$observations)
     n_obs <- length(obs)
     actions <- as.character(model$actions)
@@ -287,12 +335,13 @@ simulate_POMDP <-
       cat("\n")
     }
     
-    #bs <- replicate(n, expr = {
     ### run once for debugging
+    #bs <- replicate(n, expr = {
     #sim <- { cat("debugging on!!!\n")
-    sim <- times(n) %dopar% {
+    sim <- foreach(i = 1:n) %dopar% {
       # initialize replication
-      s <- sample(states, 1L, prob = belief)
+      alpha_vec_id <- NA_integer_
+      s <- sample.int(length(states), 1L, prob = belief)
       b <- belief
       rew <- 0
       e <- 1L
@@ -314,6 +363,18 @@ simulate_POMDP <-
       else
         visited_belief_states <- matrix(nrow = 0, ncol = 0)
       
+      if (return_trajectories)
+        trajectory <- data.frame(
+          episode = rep(NA_integer_, horizon),
+          time = rep(NA_integer_, horizon),
+          simulation_state = NA_integer_,
+          alpha_vector_id = NA_integer_,
+          a = NA_integer_,
+          r = NA_real_
+        )
+      else
+        trajectory <- NULL
+      
       for (j in 1:horizon) {
         # change matrices for time-dependent POMDPs
         if (dt) {
@@ -332,15 +393,17 @@ simulate_POMDP <-
         } else {
           if (!model$solution$converged)
             e <- .get_pg_index(model, j)
+          
+          alpha_vec_id <- which.max(model$solution$alpha[[e]] %*% b)
           a <-
-            as.integer(model$solution$pg[[e]][["action"]])[which.max(model$solution$alpha[[e]] %*% b)]
+            as.integer(model$solution$pg[[e]][["action"]])[alpha_vec_id]
         }
         
         # debug
         # cat("Episode: ", j, "\n")
         # cat("alpha: ", model$solution$alpha[[e]], "\n")
         # cat("b: ", b , "\n")
-        # cat("alpha %*% : ", model$solution$alpha[[e]] %*% b , "\n")
+        # cat("id of winning alpha vector : ", alpha_vec_id , "\n")
         # cat("a: ", a , "\n\n")
         
         s_prev <- s
@@ -353,7 +416,8 @@ simulate_POMDP <-
         obs_cnt[o] <- obs_cnt[o] + 1L
         
         # rew <- rew + rew_m[[a]][[s_prev]][s, o] * disc ^ (j - 1L)
-        rew <- rew + reward_val(model, a, s_prev, s, o, episode = current_episode) * disc ^ (j - 1L)
+        r <- reward_matrix(model, a, s_prev, s, o, episode = current_episode)
+        rew <- rew + r * disc ^ (j - 1L)
         #cat(j, ":", s_prev , "->", s, "- a:", a, "- o:", o, "- rew:", rew_m[[a]][[s_prev]][s, o], "\n")
         
         # update belief
@@ -362,6 +426,25 @@ simulate_POMDP <-
         
         if (return_beliefs)
           visited_belief_states[j,] <- b
+        
+        if (return_trajectories)
+          trajectory[j, ] <-
+          data.frame(
+            episode = i,
+            time = j - 1L,
+            simulation_state = s_prev,
+            alpha_vector_id = alpha_vec_id,
+            a = a,
+            r = r
+          )
+        
+        if(s %in% states_absorbing) {
+          if (return_trajectories)
+            trajectory <- trajectory[1:j, , drop = FALSE]
+          visited_belief_states <- visited_belief_states[1:j, , drop = FALSE]
+          # TODO: maybe add the finals state
+          break
+        }
       }
       
       # terminal values
@@ -371,30 +454,43 @@ simulate_POMDP <-
       
       rownames(visited_belief_states) <- NULL
       
-      list(
         list(
           action_cnt = action_cnt,
           state_cnt = state_cnt,
           obs_cnt = obs_cnt,
           reward = rew,
-          belief_states = visited_belief_states
-        )
+          belief_states = visited_belief_states,
+          trajectory = trajectory
       )
     }
     #, simplify = FALSE)
     
-    rew <- Reduce(c,  lapply(sim, "[[", "reward"))
-    
     time_end <- proc.time()
     if (verbose)
       print(time_end - time_start)
     
+    rew <- Reduce(c,  lapply(sim, "[[", "reward"))
+    
+    trajectories <- NULL
+    if (return_trajectories) {
+      trajectories <- Reduce(rbind, lapply(sim, "[[", "trajectory"))
+       trajectories$simulation_state <-
+         factor(trajectories$simulation_state,
+                levels = seq_along(states),
+                labels = states)
+      trajectories$a <-
+        factor(trajectories$a,
+               levels = seq_along(actions),
+               labels = actions)
+    }
+    
     list(
       avg_reward = mean(rew, na.rm = TRUE),
       action_cnt = Reduce('+', lapply(sim, "[[" , "state_cnt")),
       state_cnt =  Reduce('+', lapply(sim, "[[", "state_cnt")),
       obs_cnt =    Reduce('+', lapply(sim, "[[", "obs_cnt")),
       reward = rew,
-      belief_states = Reduce(rbind, lapply(sim, "[[", "belief_states"))
+      belief_states = Reduce(rbind, lapply(sim, "[[", "belief_states")),
+      trajectories = trajectories
     )
   }
